@@ -16,6 +16,7 @@
 # Requires Python 2.6+ and Openssl 1.0+
 #
 
+import array
 import base64
 import datetime
 import errno
@@ -26,27 +27,55 @@ import multiprocessing
 import os
 import platform
 import pwd
+import random
 import re
 import shutil
 import socket
+import string
 import struct
 import sys
 import time
+import warnings
 from pwd import getpwall
 
-import array
-
-import azurelinuxagent.common.conf as conf
-import azurelinuxagent.common.logger as logger
-import azurelinuxagent.common.utils.fileutil as fileutil
-import azurelinuxagent.common.utils.shellutil as shellutil
-import azurelinuxagent.common.utils.textutil as textutil
-
 from azurelinuxagent.common.exception import OSUtilError
-from azurelinuxagent.common.future import ustr, array_to_bytes
+
+#
+# The 'crypt' package was removed in Python 3.13.
+#
+# To work around this, WALinuxAgent 2.12 and 2.13 added a dependency on legacycrypt and imported crypt from there. From 2.14,
+# we instead get crypt from the crypt-r package. The code below needs to handle the case where the self-update WALinuxAgent
+# is running on a machine where the pre-installed WALinuxAgent is 2.12/2.13 (and crypt may be coming from legacycrypt).
+#
+# We first try importing from crypt, which may have been installed from the crypt or crypt-r packages, then try
+# importing from legacy crypt, then fallback to a dummy function that raises an exception when invoked. The Provisioning Agent
+# and JIT requests use the crypt function, so those features would fail if none of the required dependencies are installed.
+#
+try:
+    with warnings.catch_warnings():
+        warnings.filterwarnings("ignore", category=DeprecationWarning)
+        from crypt import crypt  # pylint: disable=deprecated-module
+except ImportError:
+    __CRYPT_IMPORTED__ = False
+    if sys.version_info[0] == 3 and sys.version_info[1] >= 13 or sys.version_info[0] > 3:
+        try:
+            from legacycrypt import crypt
+            __CRYPT_IMPORTED__ = True
+        except ImportError:
+            pass
+    if not __CRYPT_IMPORTED__:
+        def crypt(password, salt):
+            raise OSUtilError("This feature requires one of the 'crypt', 'legacycrypt' or 'crypt-r' Python packages to be installed.")
+
+from azurelinuxagent.common import conf
+from azurelinuxagent.common import logger
+from azurelinuxagent.common.utils import fileutil
+from azurelinuxagent.common.utils import shellutil
+from azurelinuxagent.common.utils import textutil
+
+from azurelinuxagent.common.future import ustr, array_to_bytes, UTC
 from azurelinuxagent.common.utils.cryptutil import CryptUtil
-from azurelinuxagent.common.utils.flexible_version import FlexibleVersion
-from azurelinuxagent.common.utils.networkutil import RouteEntry, NetworkInterfaceCard, AddFirewallRules
+from azurelinuxagent.common.utils.networkutil import RouteEntry, NetworkInterfaceCard
 from azurelinuxagent.common.utils.shellutil import CommandError
 
 __RULES_FILES__ = ["/lib/udev/rules.d/75-persistent-net-generator.rules",
@@ -58,58 +87,8 @@ for all distros. Each concrete distro classes could overwrite default behavior
 if needed.
 """
 
-_IPTABLES_VERSION_PATTERN = re.compile("^[^\d\.]*([\d\.]+).*$")  # pylint: disable=W1401
-_IPTABLES_LOCKING_VERSION = FlexibleVersion('1.4.21')
-
-
-def _add_wait(wait, command):
-    """
-    If 'wait' is True, adds the wait option (-w) to the given iptables command line
-    """
-    if wait:
-        command.insert(1, "-w")
-    return command
-
-
-def get_iptables_version_command():
-    return ["iptables", "--version"]
-
-
-def get_firewall_list_command(wait):
-    return _add_wait(wait, ["iptables", "-t", "security", "-L", "-nxv"])
-
-
-def get_firewall_packets_command(wait):
-    return _add_wait(wait, ["iptables", "-t", "security", "-L", "OUTPUT", "--zero", "OUTPUT", "-nxv"])
-
-
-# Precisely delete the rules created by the agent.
-# this rule was used <= 2.2.25.  This rule helped to validate our change, and determine impact.
-def get_firewall_delete_conntrack_accept_command(wait, destination):
-    return _add_wait(wait,
-                     ["iptables", "-t", "security", AddFirewallRules.DELETE_COMMAND, "OUTPUT", "-d", destination, "-p", "tcp", "-m", "conntrack",
-                      "--ctstate", "INVALID,NEW", "-j", "ACCEPT"])
-
-
-def get_delete_accept_tcp_rule(wait, destination):
-    return AddFirewallRules.get_accept_tcp_rule(AddFirewallRules.DELETE_COMMAND, destination, wait=wait)
-
-
-def get_firewall_delete_owner_accept_command(wait, destination, owner_uid):
-    return _add_wait(wait, ["iptables", "-t", "security", AddFirewallRules.DELETE_COMMAND, "OUTPUT", "-d", destination, "-p", "tcp", "-m", "owner",
-                            "--uid-owner", str(owner_uid), "-j", "ACCEPT"])
-
-
-def get_firewall_delete_conntrack_drop_command(wait, destination):
-    return _add_wait(wait,
-                     ["iptables", "-t", "security", AddFirewallRules.DELETE_COMMAND, "OUTPUT", "-d", destination, "-p", "tcp", "-m", "conntrack",
-                      "--ctstate", "INVALID,NEW", "-j", "DROP"])
-
-
-PACKET_PATTERN = "^\s*(\d+)\s+(\d+)\s+DROP\s+.*{0}[^\d]*$"  # pylint: disable=W1401
 ALL_CPUS_REGEX = re.compile('^cpu .*')
-
-_enable_firewall = True
+ALL_MEMS_REGEX = re.compile('^Mem.*')
 
 DMIDECODE_CMD = 'dmidecode --string system-uuid'
 PRODUCT_ID_FILE = '/sys/class/dmi/id/product_uuid'
@@ -122,7 +101,7 @@ IOCTL_SIOCGIFFLAGS = 0x8913
 IOCTL_SIOCGIFHWADDR = 0x8927
 IFNAMSIZ = 16
 
-IP_COMMAND_OUTPUT = re.compile('^\d+:\s+(\w+):\s+(.*)$')  # pylint: disable=W1401
+IP_COMMAND_OUTPUT = re.compile(r'^\d+:\s+([\w@]+):\s+(.*)$')
 
 STORAGE_DEVICE_PATH = '/sys/bus/vmbus/devices/'
 GEN2_DEVICE_ID = 'f8b3781a-1e82-4818-a1c3-63d806ec15bb'
@@ -148,164 +127,13 @@ class DefaultOSUtil(object):
     def get_agent_bin_path():
         return "/usr/sbin"
 
-    def get_firewall_dropped_packets(self, dst_ip=None):
-        # If a previous attempt failed, do not retry
-        global _enable_firewall  # pylint: disable=W0603
-        if not _enable_firewall:
-            return 0
-
+    @staticmethod
+    def get_vm_arch():
         try:
-            wait = self.get_firewall_will_wait()
-
-            try:
-                output = shellutil.run_command(get_firewall_packets_command(wait))
-
-                pattern = re.compile(PACKET_PATTERN.format(dst_ip))
-                for line in output.split('\n'):
-                    m = pattern.match(line)
-                    if m is not None:
-                        return int(m.group(1))
-
-            except Exception as e:
-                if isinstance(e, CommandError) and (e.returncode == 3 or e.returncode == 4):  # pylint: disable=E1101
-                    # Transient error that we ignore returncode 3. This code fires every loop
-                    # of the daemon (60m), so we will get the value eventually.
-                    # ignore returncode 4 as temporary fix (RULE_REPLACE failed (Invalid argument))
-                    return 0
-                logger.warn("Failed to get firewall packets: {0}", ustr(e))
-                return -1
-
-            return 0
-
+            return platform.machine()
         except Exception as e:
-            _enable_firewall = False
-            logger.warn("Unable to retrieve firewall packets dropped"
-                        "{0}".format(ustr(e)))
-            return -1
-
-    def get_firewall_will_wait(self):
-        # Determine if iptables will serialize access
-        try:
-            output = shellutil.run_command(get_iptables_version_command())
-        except Exception as e:
-            msg = "Unable to determine version of iptables: {0}".format(ustr(e))
-            logger.warn(msg)
-            raise Exception(msg)
-
-        m = _IPTABLES_VERSION_PATTERN.match(output)
-        if m is None:
-            msg = "iptables did not return version information: {0}".format(output)
-            logger.warn(msg)
-            raise Exception(msg)
-
-        wait = "-w" \
-            if FlexibleVersion(m.group(1)) >= _IPTABLES_LOCKING_VERSION \
-            else ""
-        return wait
-
-    def _delete_rule(self, rule):
-        """
-        Continually execute the delete operation until the return
-        code is non-zero or the limit has been reached.
-        """
-        for i in range(1, 100):  # pylint: disable=W0612
-            try:
-                rc = shellutil.run_command(rule)  # pylint: disable=W0612
-            except CommandError as e:
-                if e.returncode == 1:
-                    return
-                if e.returncode == 2:
-                    raise Exception("invalid firewall deletion rule '{0}'".format(rule))
-
-    def remove_firewall(self, dst_ip, uid, wait):
-        # If a previous attempt failed, do not retry
-        global _enable_firewall  # pylint: disable=W0603
-        if not _enable_firewall:
-            return False
-
-        try:
-            # This rule was <= 2.2.25 only, and may still exist on some VMs.  Until 2.2.25
-            # has aged out, keep this cleanup in place.
-            self._delete_rule(get_firewall_delete_conntrack_accept_command(wait, dst_ip))
-
-            self._delete_rule(get_delete_accept_tcp_rule(wait, dst_ip))
-            self._delete_rule(get_firewall_delete_owner_accept_command(wait, dst_ip, uid))
-            self._delete_rule(get_firewall_delete_conntrack_drop_command(wait, dst_ip))
-
-            return True
-
-        except Exception as e:
-            _enable_firewall = False
-            logger.info("Unable to remove firewall -- "
-                        "no further attempts will be made: "
-                        "{0}".format(ustr(e)))
-            return False
-
-    def remove_legacy_firewall_rule(self, dst_ip):
-        # This function removes the legacy firewall rule that was added <= 2.2.25.
-        # Not adding the global _enable_firewall check here as this will only be called once per service start and
-        # we dont want the state of this call to affect other iptable calls.
-        try:
-            wait = self.get_firewall_will_wait()
-
-            # This rule was <= 2.2.25 only, and may still exist on some VMs.  Until 2.2.25
-            # has aged out, keep this cleanup in place.
-            self._delete_rule(get_firewall_delete_conntrack_accept_command(wait, dst_ip))
-        except Exception as error:
-            logger.info(
-                "Unable to remove legacy firewall rule, won't try removing it again. Error: {0}".format(ustr(error)))
-
-    def enable_firewall(self, dst_ip, uid):
-        """
-        It checks if every iptable rule exists and add them if not present. It returns a tuple(enable firewall success status, update rules flag)
-        enable firewall success status: Returns True if every firewall rule exists otherwise False
-        update rules flag: Returns True if rules are updated otherwise False
-        """
-        # This is to send telemetry when iptable rules updated
-        is_firewall_rules_updated = False
-        # If a previous attempt failed, do not retry
-        global _enable_firewall  # pylint: disable=W0603
-        if not _enable_firewall:
-            return False, is_firewall_rules_updated
-
-        try:
-            wait = self.get_firewall_will_wait()
-
-            # check every iptable rule and delete others if any rule is missing
-            #   and append every iptable rule to the end of the chain.
-            try:
-                if not AddFirewallRules.verify_iptables_rules_exist(wait, dst_ip, uid):
-                    self.remove_firewall(dst_ip, uid, wait)
-                    AddFirewallRules.add_iptables_rules(wait, dst_ip, uid)
-                    is_firewall_rules_updated = True
-            except CommandError as e:
-                if e.returncode == 2:
-                    self.remove_firewall(dst_ip, uid, wait)
-                    msg = "please upgrade iptables to a version that supports the -C option"
-                    logger.warn(msg)
-                    raise
-            except Exception as error:
-                logger.warn(ustr(error))
-                raise
-
-            return True, is_firewall_rules_updated
-
-        except Exception as e:
-            _enable_firewall = False
-            logger.info("Unable to establish firewall -- "
-                        "no further attempts will be made: "
-                        "{0}".format(ustr(e)))
-            return False, is_firewall_rules_updated
-
-    def get_firewall_list(self, wait=None):
-        try:
-            if wait is None:
-                wait = self.get_firewall_will_wait()
-            output = shellutil.run_command(get_firewall_list_command(wait))
-            return output
-        except Exception as e:
-            logger.warn("Listing firewall rules failed: {0}".format(ustr(e)))
-            return ""
+            logger.warn("Unable to determine cpu architecture: {0}", ustr(e))
+            return "unknown"
 
     @staticmethod
     def _correct_instance_id(instance_id):
@@ -373,6 +201,9 @@ class DefaultOSUtil(object):
         except KeyError:
             return None
 
+    def get_root_username(self):
+        return "root"
+
     def is_sys_user(self, username):
         """
         Check whether use is a system user. 
@@ -421,11 +252,21 @@ class DefaultOSUtil(object):
         if self.is_sys_user(username):
             raise OSUtilError(("User {0} is a system user, "
                                "will not set password.").format(username))
-        passwd_hash = textutil.gen_password_hash(password, crypt_id, salt_len)
+        passwd_hash = DefaultOSUtil.gen_password_hash(password, crypt_id, salt_len)
 
         self._run_command_raising_OSUtilError(["usermod", "-p", passwd_hash, username],
                                               err_msg="Failed to set password for {0}".format(username))
 
+    @staticmethod
+    def gen_password_hash(password, crypt_id, salt_len):
+        collection = string.ascii_letters + string.digits
+        salt = ''.join(random.choice(collection) for _ in range(salt_len))
+        salt = "${0}${1}".format(crypt_id, salt)
+        if sys.version_info[0] == 2:
+            # if python 2.*, encode to type 'str' to prevent Unicode Encode Error from crypt.crypt
+            password = password.encode('utf-8')
+        return crypt(password, salt)
+
     def get_users(self):
         return getpwall()
 
@@ -524,7 +365,7 @@ class DefaultOSUtil(object):
                 raise OSUtilError("Bad public key: {0}".format(value))
             if not value.endswith("\n"):
                 value += "\n"
-            fileutil.write_file(path, value)
+            fileutil.write_file(path, value, append=True)
         elif thumbprint is not None:
             lib_dir = conf.get_lib_dir()
             crt_path = os.path.join(lib_dir, thumbprint + '.crt')
@@ -1025,8 +866,8 @@ class DefaultOSUtil(object):
                         else:
                             try:
                                 expire_string = line.split(" ", 4)[-1].strip(";")
-                                expire_date = datetime.datetime.strptime(expire_string, FORMAT_DATETIME)
-                                if expire_date > datetime.datetime.utcnow():
+                                expire_date = datetime.datetime.strptime(expire_string, FORMAT_DATETIME).replace(tzinfo=UTC)
+                                if expire_date > datetime.datetime.now(UTC):
                                     expired = False
                             except:  # pylint: disable=W0702
                                 logger.error("could not parse expiry token '{0}'".format(line))
@@ -1126,7 +967,7 @@ class DefaultOSUtil(object):
         Add specified route 
         """
         try:
-            cmd = ["ip", "route", "add", net, "via", gateway]
+            cmd = ["ip", "route", "add", str(net), "via", gateway]
             return shellutil.run_command(cmd)
         except CommandError:
             return ""
@@ -1136,9 +977,12 @@ class DefaultOSUtil(object):
         return [int(n) for n in text.split()]
 
     @staticmethod
-    def _get_dhcp_pid(command):
+    def _get_dhcp_pid(command, transform_command_output=None):
         try:
-            return DefaultOSUtil._text_to_pid_list(shellutil.run_command(command))
+            output = shellutil.run_command(command)
+            if transform_command_output is not None:
+                output = transform_command_output(output)
+            return DefaultOSUtil._text_to_pid_list(output)
         except CommandError as exception:  # pylint: disable=W0612
             return []
 
@@ -1175,11 +1019,20 @@ class DefaultOSUtil(object):
             else:
                 logger.warn("exceeded restart retries")
 
-    def publish_hostname(self, hostname):
+    def check_and_recover_nic_state(self, ifname):
+        # TODO: This should be implemented for all distros where we reset the network during publishing hostname. Currently it is only implemented in RedhatOSUtil.
+        pass
+
+    def publish_hostname(self, hostname, recover_nic=False):
+        """
+        Publishes the provided hostname.
+        """
         self.set_dhcp_hostname(hostname)
         self.set_hostname_record(hostname)
         ifname = self.get_if_name()
         self.restart_if(ifname)
+        if recover_nic:
+            self.check_and_recover_nic_state(ifname)
 
     def set_scsi_disks_timeout(self, timeout):
         for dev in os.listdir("/sys/block"):
@@ -1411,6 +1264,27 @@ class DefaultOSUtil(object):
                     break
         return system_cpu
 
+    @staticmethod
+    def get_used_and_available_system_memory():
+        """
+        Get the contents of free -b in bytes.
+        # free -b
+        #              total        used        free      shared  buff/cache   available
+        # Mem:     8340144128   619352064  5236809728     1499136  2483982336  7426314240
+        # Swap:             0           0           0
+
+        :return: used and available memory in megabytes
+        """
+        used_mem = available_mem = 0
+        free_cmd = ["free", "-b"]
+        memory = shellutil.run_command(free_cmd)
+        for line in memory.split("\n"):
+            if ALL_MEMS_REGEX.match(line):
+                mems = line.split()
+                used_mem = int(mems[2])
+                available_mem = int(mems[6])  # see "man free" for a description of these fields
+        return used_mem/(1024 ** 2), available_mem/(1024 ** 2)
+
     def get_nic_state(self, as_string=False):
         """
         Capture NIC state (IPv4 and IPv6 addresses plus link state).
