@@ -20,6 +20,7 @@ import base64
 import datetime
 import json
 import os.path
+import threading
 import uuid
 
 from azurelinuxagent.common import logger, conf
@@ -27,7 +28,7 @@ from azurelinuxagent.common.errorstate import ErrorState, ERROR_STATE_HOST_PLUGI
 from azurelinuxagent.common.event import WALAEventOperation, add_event
 from azurelinuxagent.common.exception import HttpError, ProtocolError, ResourceGoneError
 from azurelinuxagent.common.utils.flexible_version import FlexibleVersion
-from azurelinuxagent.common.future import ustr, httpclient
+from azurelinuxagent.common.future import ustr, httpclient, UTC, datetime_min_utc
 from azurelinuxagent.common.protocol.healthservice import HealthService
 from azurelinuxagent.common.protocol.extensions_goal_state import VmSettingsParseError, GoalStateSource
 from azurelinuxagent.common.protocol.extensions_goal_state_factory import ExtensionsGoalStateFactory
@@ -55,6 +56,7 @@ _HEADER_VERSION = "x-ms-version"
 _HEADER_HOST_CONFIG_NAME = "x-ms-host-config-name"
 _HEADER_ARTIFACT_LOCATION = "x-ms-artifact-location"
 _HEADER_ARTIFACT_MANIFEST_LOCATION = "x-ms-artifact-manifest-location"
+_HEADER_VERIFY_FROM_ARTIFACTS_BLOB = "x-ms-verify-from-artifacts-blob"
 
 MAXIMUM_PAGEBLOB_PAGE_SIZE = 4 * 1024 * 1024  # Max page size: 4MB
 
@@ -87,19 +89,13 @@ class HostPluginProtocol(object):
         self.status_last_timestamp = None
         self._version = FlexibleVersion("0.0.0.0")  # Version 0 means "unknown"
         self._supports_vm_settings = None   # Tri-state variable: None == Not Initialized, True == Supports, False == Does Not Support
-        self._supports_vm_settings_next_check = datetime.datetime.now()
+        self._supports_vm_settings_next_check = datetime.datetime.now(UTC)
         self._vm_settings_error_reporter = _VmSettingsErrorReporter()
         self._cached_vm_settings = None  # Cached value of the most recent vmSettings
 
         # restore the state of Fast Track
-        if not os.path.exists(self._get_fast_track_state_file()):
-            self._supports_vm_settings = False
-            self._supports_vm_settings_next_check = datetime.datetime.now()
-            self._fast_track_timestamp = timeutil.create_timestamp(datetime.datetime.min)
-        else:
-            self._supports_vm_settings = True
-            self._supports_vm_settings_next_check = datetime.datetime.now()
-            self._fast_track_timestamp = HostPluginProtocol.get_fast_track_timestamp()
+        self._supports_vm_settings = os.path.exists(self._get_fast_track_state_file())
+        self._fast_track_timestamp = HostPluginProtocol.get_fast_track_timestamp()
 
     @staticmethod
     def _extract_deployment_id(role_config_name):
@@ -183,19 +179,21 @@ class HostPluginProtocol(object):
 
         return url, headers
 
-    def get_artifact_request(self, artifact_url, artifact_manifest_url=None):
+    def get_artifact_request(self, artifact_url, use_verify_header, artifact_manifest_url=None):
         if not self.ensure_initialized():
             raise ProtocolError("HostGAPlugin: Host plugin channel is not available")
 
         if textutil.is_str_none_or_whitespace(artifact_url):
             raise ProtocolError("HostGAPlugin: No extension artifact url was provided")
 
-        url = URI_FORMAT_GET_EXTENSION_ARTIFACT.format(self.endpoint,
-                                                       HOST_PLUGIN_PORT)
-        headers = {_HEADER_VERSION: API_VERSION,
-                   _HEADER_CONTAINER_ID: self.container_id,
-                   _HEADER_HOST_CONFIG_NAME: self.role_config_name,
-                   _HEADER_ARTIFACT_LOCATION: artifact_url}
+        url = URI_FORMAT_GET_EXTENSION_ARTIFACT.format(self.endpoint, HOST_PLUGIN_PORT)
+        headers = {
+            _HEADER_VERSION: API_VERSION,
+               _HEADER_CONTAINER_ID: self.container_id,
+               _HEADER_HOST_CONFIG_NAME: self.role_config_name,
+               _HEADER_ARTIFACT_LOCATION: artifact_url}
+        if use_verify_header:
+            headers[_HEADER_VERIFY_FROM_ARTIFACTS_BLOB] = "true"
 
         if artifact_manifest_url is not None:
             headers[_HEADER_ARTIFACT_MANIFEST_LOCATION] = artifact_manifest_url
@@ -211,7 +209,7 @@ class HostPluginProtocol(object):
                               self.fetch_error_state,
                               self.fetch_last_timestamp,
                               HostPluginProtocol.FETCH_REPORTING_PERIOD):
-            self.fetch_last_timestamp = datetime.datetime.utcnow()
+            self.fetch_last_timestamp = datetime.datetime.now(UTC)
             health_signal = self.fetch_error_state.is_triggered() is False
             self.health_service.report_host_plugin_extension_artifact(is_healthy=health_signal,
                                                                       source=source,
@@ -222,7 +220,7 @@ class HostPluginProtocol(object):
                               self.status_error_state,
                               self.status_last_timestamp,
                               HostPluginProtocol.STATUS_REPORTING_PERIOD):
-            self.status_last_timestamp = datetime.datetime.utcnow()
+            self.status_last_timestamp = datetime.datetime.now(UTC)
             health_signal = self.status_error_state.is_triggered() is False
             self.health_service.report_host_plugin_status(is_healthy=health_signal,
                                                           response=response)
@@ -247,9 +245,9 @@ class HostPluginProtocol(object):
             error_state.incr()
 
         if last_timestamp is None:
-            last_timestamp = datetime.datetime.utcnow() - period
+            last_timestamp = datetime.datetime.now(UTC) - period
 
-        return datetime.datetime.utcnow() >= (last_timestamp + period)
+        return datetime.datetime.now(UTC) >= (last_timestamp + period)
 
     def put_vm_log(self, content):
         """
@@ -266,7 +264,8 @@ class HostPluginProtocol(object):
         response = restutil.http_put(url,
                                      data=content,
                                      headers=self._build_log_headers(),
-                                     redact_data=True)
+                                     redact_data=True,
+                                     timeout=30)
 
         if restutil.request_failed(response):
             error_response = restutil.read_response_error(response)
@@ -419,19 +418,24 @@ class HostPluginProtocol(object):
         # This file keeps the timestamp of the most recent goal state if it was retrieved via Fast Track
         return os.path.join(conf.get_lib_dir(), "fast_track.json")
 
+    # Multiple threads create instances of HostPluginProtocol; we use this lock to protect access to the state file for Fast Track
+    _fast_track_state_lock = threading.RLock()
+
     @staticmethod
     def _save_fast_track_state(timestamp):
         try:
-            with open(HostPluginProtocol._get_fast_track_state_file(), "w") as file_:
-                json.dump({"timestamp": timestamp}, file_)
+            with HostPluginProtocol._fast_track_state_lock:
+                with open(HostPluginProtocol._get_fast_track_state_file(), "w") as file_:
+                    json.dump({"timestamp": timestamp}, file_)
         except Exception as e:
             logger.warn("Error updating the Fast Track state ({0}): {1}", HostPluginProtocol._get_fast_track_state_file(), ustr(e))
 
     @staticmethod
     def clear_fast_track_state():
         try:
-            if os.path.exists(HostPluginProtocol._get_fast_track_state_file()):
-                os.remove(HostPluginProtocol._get_fast_track_state_file())
+            with HostPluginProtocol._fast_track_state_lock:
+                if os.path.exists(HostPluginProtocol._get_fast_track_state_file()):
+                    os.remove(HostPluginProtocol._get_fast_track_state_file())
         except Exception as e:
             logger.warn("Error clearing the current state for Fast Track ({0}): {1}", HostPluginProtocol._get_fast_track_state_file(),
                         ustr(e))
@@ -439,19 +443,20 @@ class HostPluginProtocol(object):
     @staticmethod
     def get_fast_track_timestamp():
         """
-        Returns the timestamp of the most recent FastTrack goal state retrieved by fetch_vm_settings(), or None if the most recent
-        goal state was Fabric or fetch_vm_settings() has not been invoked.
+        Returns the timestamp of the most recent FastTrack goal state retrieved by fetch_vm_settings(), or a timestamp representing datetime.min if
+        the most recent goal state was Fabric or fetch_vm_settings() has not been invoked.
         """
-        if not os.path.exists(HostPluginProtocol._get_fast_track_state_file()):
-            return timeutil.create_timestamp(datetime.datetime.min)
+        with HostPluginProtocol._fast_track_state_lock:
+            state_file = HostPluginProtocol._get_fast_track_state_file()
+            if not os.path.exists(state_file):
+                return timeutil.create_utc_timestamp(datetime_min_utc)
 
-        try:
-            with open(HostPluginProtocol._get_fast_track_state_file(), "r") as file_:
-                return json.load(file_)["timestamp"]
-        except Exception as e:
-            logger.warn("Can't retrieve the timestamp for the most recent Fast Track goal state ({0}), will assume the current time. Error: {1}",
-                    HostPluginProtocol._get_fast_track_state_file(), ustr(e))
-        return timeutil.create_timestamp(datetime.datetime.utcnow())
+            try:
+                with open(state_file, "r") as file_:
+                    return json.load(file_)["timestamp"]
+            except Exception as e:
+                logger.warn("Can't retrieve the timestamp for the most recent Fast Track goal state ({0}), will assume the current time. Error: {1}", state_file, ustr(e))
+            return timeutil.create_utc_timestamp(datetime.datetime.now(UTC))
 
     def fetch_vm_settings(self, force_update=False):
         """
@@ -479,7 +484,7 @@ class HostPluginProtocol(object):
                     raise VmSettingsNotSupported()
             finally:
                 self._supports_vm_settings = False
-                self._supports_vm_settings_next_check = datetime.datetime.now() + datetime.timedelta(hours=6)  # check again in 6 hours
+                self._supports_vm_settings_next_check = datetime.datetime.now(UTC) + datetime.timedelta(hours=6)  # check again in 6 hours
 
         def format_message(msg):
             return "GET vmSettings [correlation ID: {0} eTag: {1}]: {2}".format(correlation_id, etag, msg)
@@ -487,7 +492,7 @@ class HostPluginProtocol(object):
         try:
             # Raise if VmSettings are not supported, but check again periodically since the HostGAPlugin could have been updated since the last check
             # Note that self._host_plugin_supports_vm_settings can be None, so we need to compare against False
-            if self._supports_vm_settings == False and self._supports_vm_settings_next_check > datetime.datetime.now():
+            if not self._supports_vm_settings and self._supports_vm_settings_next_check > datetime.datetime.now(UTC):
                 # Raise VmSettingsNotSupported directly instead of using raise_not_supported() to avoid resetting the timestamp for the next check
                 raise VmSettingsNotSupported()
 
@@ -547,8 +552,8 @@ class HostPluginProtocol(object):
                 logger.info(message)
                 add_event(op=WALAEventOperation.HostPlugin, message=message, is_success=True)
 
-            # Don't support HostGAPlugin versions older than 124
-            if vm_settings.host_ga_plugin_version < FlexibleVersion("1.0.8.124"):
+            # Don't support HostGAPlugin versions older than 133
+            if vm_settings.host_ga_plugin_version < FlexibleVersion("1.0.8.133"):
                 raise_not_supported()
 
             self._supports_vm_settings = True
@@ -615,7 +620,7 @@ class _VmSettingsErrorReporter(object):
         self._request_failure_count = 0  # Total count of requests that could not be issued (does not include timeouts or requests that were actually issued and failed, for example, with 500 or 400 statuses)
         self._server_error_count = 0  # Count of server side errors (HTTP status in the 500s)
         self._timeout_count = 0  # Count of timeouts on vmSettings requests
-        self._next_period = datetime.datetime.now() + _VmSettingsErrorReporter._Period
+        self._next_period = datetime.datetime.now(UTC) + _VmSettingsErrorReporter._Period
 
     def report_request(self):
         self._request_count += 1
@@ -624,7 +629,7 @@ class _VmSettingsErrorReporter(object):
         self._error_count += 1
 
         if self._error_count <= _VmSettingsErrorReporter._MaxErrors:
-            add_event(op=WALAEventOperation.VmSettings, message="[{0}] {1}".format(category, error), is_success=False, log_event=False)
+            add_event(op=WALAEventOperation.VmSettings, message="[{0}] {1}".format(category, error), is_success=True, log_event=False)
 
         if category == _VmSettingsError.ClientError:
             self._client_error_count += 1
@@ -638,7 +643,7 @@ class _VmSettingsErrorReporter(object):
             self._timeout_count += 1
 
     def report_summary(self):
-        if datetime.datetime.now() >= self._next_period:
+        if datetime.datetime.now(UTC) >= self._next_period:
             summary = {
                 "requests":       self._request_count,
                 "errors":         self._error_count,
@@ -648,7 +653,7 @@ class _VmSettingsErrorReporter(object):
                 "failedRequests": self._request_failure_count
             }
             message = json.dumps(summary)
-            add_event(op=WALAEventOperation.VmSettingsSummary, message=message, is_success=False, log_event=False)
+            add_event(op=WALAEventOperation.VmSettingsSummary, message=message, is_success=True, log_event=False)
             if self._error_count > 0:
                 logger.info("[VmSettingsSummary] {0}", message)
 
