@@ -108,6 +108,7 @@ class TestCGroupsTelemetry(AgentTestCase):
     def _track_new_extension_cgroup_controllers(num_extensions):
         for i in range(num_extensions):
             dummy_cpu_controller = CpuControllerV1("dummy_extension_{0}".format(i), "dummy_cpu_path_{0}".format(i))
+            dummy_cpu_controller.track_throttle_time(True)
             CGroupsTelemetry.track_cgroup_controller(dummy_cpu_controller)
 
             dummy_memory_controller = MemoryControllerV1("dummy_extension_{0}".format(i), "dummy_memory_path_{0}".format(i))
@@ -115,15 +116,17 @@ class TestCGroupsTelemetry(AgentTestCase):
 
     def _assert_cgroup_controllers_are_tracked(self, num_extensions):
         for i in range(num_extensions):
-            self.assertTrue(CGroupsTelemetry.is_tracked("dummy_cpu_path_{0}".format(i)))
-            self.assertTrue(CGroupsTelemetry.is_tracked("dummy_memory_path_{0}".format(i)))
+            self.assertTrue(CGroupsTelemetry.is_tracked("cpu:dummy_cpu_path_{0}".format(i)))
+            self.assertTrue(CGroupsTelemetry.is_tracked("memory:dummy_memory_path_{0}".format(i)))
 
-    def _assert_polled_metrics_equal(self, metrics, cpu_metric_value, current_total_memory_metric_value, current_anon_memory_metric_value, current_cache_memory_metric_value, max_memory_metric_value, swap_memory_value):
+    def _assert_polled_metrics_equal(self, metrics, cpu_processor_metric_value, cpu_throttled_metric_value, current_total_memory_metric_value, current_anon_memory_metric_value, current_cache_memory_metric_value, max_memory_metric_value, swap_memory_value):
         for metric in metrics:
             self.assertIn(metric.category, ["CPU", "Memory"])
             if metric.category == "CPU":
-                self.assertEqual(metric.counter, "% Processor Time")
-                self.assertEqual(metric.value, cpu_metric_value)
+                if metric.counter == MetricsCounter.PROCESSOR_PERCENT_TIME:
+                    self.assertEqual(metric.value, cpu_processor_metric_value)
+                if metric.counter == MetricsCounter.THROTTLED_TIME:
+                    self.assertEqual(metric.value, cpu_throttled_metric_value)
             if metric.category == "Memory":
                 self.assertIn(metric.counter, [MetricsCounter.TOTAL_MEM_USAGE, MetricsCounter.ANON_MEM_USAGE, MetricsCounter.CACHE_MEM_USAGE, MetricsCounter.MAX_MEM_USAGE, MetricsCounter.SWAP_MEM_USAGE])
                 if metric.counter == MetricsCounter.TOTAL_MEM_USAGE:
@@ -145,30 +148,33 @@ class TestCGroupsTelemetry(AgentTestCase):
         with patch("azurelinuxagent.ga.memorycontroller.MemoryControllerV1.get_max_memory_usage") as patch_get_memory_max_usage:
             with patch("azurelinuxagent.ga.memorycontroller.MemoryControllerV1.get_memory_usage") as patch_get_memory_usage:
                 with patch("azurelinuxagent.ga.memorycontroller.MemoryControllerV1.try_swap_memory_usage") as patch_try_swap_memory_usage:
-                    with patch("azurelinuxagent.ga.cpucontroller.CpuControllerV1.get_cpu_usage") as patch_get_cpu_usage:
-                        with patch("azurelinuxagent.ga.cgroupcontroller._CgroupController.is_active") as patch_is_active:
-                            patch_is_active.return_value = True
+                    with patch("azurelinuxagent.ga.cpucontroller.CpuControllerV1.get_cpu_throttled_time") as patch_get_throttle_time:
+                        with patch("azurelinuxagent.ga.cpucontroller.CpuControllerV1.get_cpu_usage") as patch_get_cpu_usage:
+                            with patch("azurelinuxagent.ga.cgroupcontroller._CgroupController.is_active") as patch_is_active:
+                                patch_is_active.return_value = True
 
-                            current_cpu = 30
-                            current_anon_memory = 209715200
-                            current_cache_memory = 314572800
-                            current_total_memory = 209715200 + 314572800
-                            current_max_memory = 471859200
-                            current_swap_memory = 20971520
+                                current_cpu = 30
+                                current_throttle_time = 5
+                                current_anon_memory = 209715200
+                                current_cache_memory = 314572800
+                                current_total_memory = 209715200 + 314572800
+                                current_max_memory = 471859200
+                                current_swap_memory = 20971520
 
-                            # 1 CPU metric + 1 total Memory + 1 anon memory + 1 cache memory + 1 Max memory + 1 swap memory
-                            num_of_metrics_per_extn_expected = 6
-                            patch_get_cpu_usage.return_value = current_cpu
-                            patch_get_memory_usage.return_value = current_anon_memory, current_cache_memory  # example 200 MB, 300 MB
-                            patch_get_memory_max_usage.return_value = current_max_memory  # example 450 MB
-                            patch_try_swap_memory_usage.return_value = current_swap_memory  # example 20MB
-                            num_polls = 18
+                                # 1 CPU metric + 1 CPU throttle + 1 total Memory + 1 anon memory + 1 cache memory + 1 Max memory + 1 swap memory
+                                num_of_metrics_per_extn_expected = 7
+                                patch_get_cpu_usage.return_value = current_cpu
+                                patch_get_throttle_time.return_value = current_throttle_time
+                                patch_get_memory_usage.return_value = current_anon_memory, current_cache_memory  # example 200 MB, 300 MB
+                                patch_get_memory_max_usage.return_value = current_max_memory  # example 450 MB
+                                patch_try_swap_memory_usage.return_value = current_swap_memory  # example 20MB
+                                num_polls = 18
 
-                            for data_count in range(1, num_polls + 1):  # pylint: disable=unused-variable
-                                metrics = CGroupsTelemetry.poll_all_tracked()
+                                for data_count in range(1, num_polls + 1):  # pylint: disable=unused-variable
+                                    metrics = CGroupsTelemetry.poll_all_tracked()
 
-                                self.assertEqual(len(metrics), num_extensions * num_of_metrics_per_extn_expected)
-                                self._assert_polled_metrics_equal(metrics, current_cpu, current_total_memory, current_anon_memory, current_cache_memory, current_max_memory, current_swap_memory)
+                                    self.assertEqual(len(metrics), num_extensions * num_of_metrics_per_extn_expected)
+                                    self._assert_polled_metrics_equal(metrics, current_cpu, current_throttle_time, current_total_memory, current_anon_memory, current_cache_memory, current_max_memory, current_swap_memory)
 
     @patch("azurelinuxagent.ga.memorycontroller.MemoryControllerV1.get_max_memory_usage", side_effect=raise_ioerror)
     @patch("azurelinuxagent.ga.memorycontroller.MemoryControllerV1.get_memory_usage", side_effect=raise_ioerror)
@@ -285,15 +291,16 @@ class TestCGroupsTelemetry(AgentTestCase):
     @patch("azurelinuxagent.ga.memorycontroller.MemoryControllerV1.try_swap_memory_usage")
     @patch("azurelinuxagent.ga.memorycontroller.MemoryControllerV1.get_max_memory_usage")
     @patch("azurelinuxagent.ga.memorycontroller.MemoryControllerV1.get_memory_usage")
+    @patch("azurelinuxagent.ga.cpucontroller.CpuControllerV1.get_cpu_throttled_time")
     @patch("azurelinuxagent.ga.cpucontroller.CpuControllerV1.get_cpu_usage")
     @patch("azurelinuxagent.ga.cgroupcontroller._CgroupController.is_active")
-    def test_telemetry_calculations(self, patch_is_active, patch_get_cpu_usage, patch_get_memory_usage, patch_get_memory_max_usage, patch_try_memory_swap_usage,
+    def test_telemetry_calculations(self, patch_is_active, patch_get_cpu_usage, patch_get_throttle_usage, patch_get_memory_usage, patch_get_memory_max_usage, patch_try_memory_swap_usage,
                                     *args):  # pylint: disable=unused-argument
         num_polls = 10
         num_extensions = 1
 
         cpu_percent_values = [random.randint(0, 100) for _ in range(num_polls)]
-
+        cpu_throttle_values = [random.randint(0, 100) for _ in range(num_polls)]
         # only verifying calculations and not validity of the values.
         anon_usage_values = [random.randint(0, 8 * 1024 ** 3) for _ in range(num_polls)]
         cache_usage_values = [random.randint(0, 8 * 1024 ** 3) for _ in range(num_polls)]
@@ -306,15 +313,16 @@ class TestCGroupsTelemetry(AgentTestCase):
         for i in range(num_polls):
             patch_is_active.return_value = True
             patch_get_cpu_usage.return_value = cpu_percent_values[i]
+            patch_get_throttle_usage.return_value = cpu_throttle_values[i]
             patch_get_memory_usage.return_value = anon_usage_values[i], cache_usage_values[i]
             patch_get_memory_max_usage.return_value = max_memory_usage_values[i]
             patch_try_memory_swap_usage.return_value = swap_usage_values[i]
 
             metrics = CGroupsTelemetry.poll_all_tracked()
 
-            # 1 CPU metric + 1 Total Memory + 1 anon memory + 1 cache memory + 1 Max memory + 1 swap memory
-            self.assertEqual(len(metrics), 6 * num_extensions)
-            self._assert_polled_metrics_equal(metrics, cpu_percent_values[i], anon_usage_values[i] + cache_usage_values[i], anon_usage_values[i], cache_usage_values[i], max_memory_usage_values[i], swap_usage_values[i])
+            # 1 CPU Processor + 1 CPU Throttle + 1 Total Memory + 1 anon memory + 1 cache memory + 1 Max memory + 1 swap memory
+            self.assertEqual(len(metrics), 7 * num_extensions)
+            self._assert_polled_metrics_equal(metrics, cpu_percent_values[i], cpu_throttle_values[i], anon_usage_values[i] + cache_usage_values[i], anon_usage_values[i], cache_usage_values[i], max_memory_usage_values[i], swap_usage_values[i])
 
     def test_cgroup_tracking(self, *args):  # pylint: disable=unused-argument
         num_extensions = 5
@@ -347,8 +355,8 @@ class TestCGroupsTelemetry(AgentTestCase):
                 for data_count in range(poll_count, 10):  # pylint: disable=unused-variable
                     metrics = CGroupsTelemetry.poll_all_tracked()
 
-                    self.assertEqual(len(metrics), num_extensions * 1)  # Only CPU populated
-                    self._assert_polled_metrics_equal(metrics, current_cpu, 0, 0, 0, 0, 0)
+                    self.assertEqual(len(metrics), num_extensions * 2)  # only CPU populated (processor time and throttled time)
+                    self._assert_polled_metrics_equal(metrics, current_cpu, 0,0, 0, 0, 0, 0)
 
     @patch("azurelinuxagent.ga.cpucontroller.CpuControllerV1.get_cpu_usage", side_effect=raise_ioerror)
     def test_process_cgroup_metric_with_no_cpu_cgroup_mounted(self, *args):  # pylint: disable=unused-argument
@@ -376,7 +384,7 @@ class TestCGroupsTelemetry(AgentTestCase):
                             metrics = CGroupsTelemetry.poll_all_tracked()
                             # Memory is only populated, CPU is not. Thus 5 metrics for memory.
                             self.assertEqual(len(metrics), num_extensions * 5)
-                            self._assert_polled_metrics_equal(metrics, 0, current_total_memory, current_anon_memory, current_cache_memory, current_max_memory, current_swap_memory)
+                            self._assert_polled_metrics_equal(metrics, 0, 0, current_total_memory, current_anon_memory, current_cache_memory, current_max_memory, current_swap_memory)
 
     @patch("azurelinuxagent.ga.memorycontroller.MemoryControllerV1.get_memory_usage", side_effect=raise_ioerror)
     @patch("azurelinuxagent.ga.memorycontroller.MemoryControllerV1.get_max_memory_usage", side_effect=raise_ioerror)
@@ -408,6 +416,7 @@ class TestCGroupsTelemetry(AgentTestCase):
         cpu_throttled_values = [random.randint(0, 60 * 60) for _ in range(num_polls)]
 
         dummy_cpu_cgroup = CpuControllerV1("dummy_extension_name", "dummy_cpu_path")
+        dummy_cpu_cgroup.track_throttle_time(True)
         CGroupsTelemetry.track_cgroup_controller(dummy_cpu_cgroup)
         self.assertEqual(1, len(CGroupsTelemetry._tracked))
 
