@@ -46,6 +46,7 @@ from lisa.sut_orchestrator.azure.platform_ import AzurePlatform  # pylint: disab
 
 import makepkg
 from azurelinuxagent.common.version import AGENT_VERSION
+from azurelinuxagent.common.future import UTC, datetime_min_utc, datetime_max_utc
 from tests_e2e.tests.lib.retry import retry_if_false
 
 from tests_e2e.tests.lib.virtual_machine_client import VirtualMachineClient
@@ -229,6 +230,8 @@ class AgentTestSuite(LisaTestSuite):
         self._skip_setup = variables["skip_setup"]
         self._keep_environment = variables["keep_environment"]
         self._collect_logs = variables["collect_logs"]
+        if self._collect_logs not in [CollectLogs.Always, CollectLogs.Failed, CollectLogs.No]:
+            raise ValueError(f"Invalid value for the collect_logs parameter: {self._collect_logs}")
 
         # The AgentTestSuiteCombinator can create 4 kinds of platform/environment combinations:
         #
@@ -554,7 +557,7 @@ class AgentTestSuite(LisaTestSuite):
         with set_thread_name(self._environment_name):
             log_path: Path = self._log_path / f"env-{self._environment_name}.log"
             with set_current_thread_log(log_path):
-                start_time: datetime.datetime = datetime.datetime.now()
+                start_time: datetime.datetime = datetime.datetime.now(UTC)
                 failed_cases = []
 
                 try:
@@ -577,7 +580,7 @@ class AgentTestSuite(LisaTestSuite):
                                 test_suite_success = False
                                 raise
 
-                        check_log_start_time = datetime.datetime.min
+                        check_log_start_time = datetime_min_utc
 
                         for suite in self._test_suites:
                             log.info("Executing test suite %s", suite.name)
@@ -624,8 +627,8 @@ class AgentTestSuite(LisaTestSuite):
         """
         suite_name = suite.name
         suite_full_name = f"{suite_name}-{self._environment_name}"
-        suite_start_time: datetime.datetime = datetime.datetime.now()
-        check_log_start_time_override = datetime.datetime.max  # tests can override the timestamp for the agent log check with the get_ignore_errors_before_timestamp() method
+        suite_start_time: datetime.datetime = datetime.datetime.now(UTC)
+        check_log_start_time_override = datetime_max_utc  # tests can override the timestamp for the agent log check with the get_ignore_errors_before_timestamp() method
 
         with set_thread_name(suite_full_name):  # The thread name is added to the LISA log
             log_path: Path = self._log_path / f"{suite_full_name}.log"
@@ -642,7 +645,7 @@ class AgentTestSuite(LisaTestSuite):
 
                     for test in suite.tests:
                         test_full_name = f"{suite_name}-{test.name}"
-                        test_start_time: datetime.datetime = datetime.datetime.now()
+                        test_start_time: datetime.datetime = datetime.datetime.now(UTC)
 
                         log.info("******** Executing %s", test.name)
                         self._lisa_log.info("Executing test %s", test_full_name)
@@ -715,7 +718,7 @@ class AgentTestSuite(LisaTestSuite):
                         # Check if the test is requesting to override the timestamp for the agent log check.
                         # Note that if multiple tests in the suite provide an override, we'll use the earliest timestamp.
                         test_check_log_start_time = test_instance.get_ignore_errors_before_timestamp()
-                        if test_check_log_start_time != datetime.datetime.min:
+                        if test_check_log_start_time != datetime_min_utc:
                             check_log_start_time_override = min(check_log_start_time_override, test_check_log_start_time)
 
                         if not test_success and test.blocks_suite:
@@ -729,21 +732,21 @@ class AgentTestSuite(LisaTestSuite):
                         log.info("\t%s", r)
                     log.info("")
 
-                except:  # pylint: disable=bare-except
+                except Exception as e:
                     suite_success = False
                     self._report_test_result(
                         suite_full_name,
                         suite_name,
                         TestStatus.FAILED,
                         suite_start_time,
-                        message=f"Unhandled exception while executing test suite {suite_name}.",
+                        message=f"Unhandled exception while executing test suite {suite_name}: {e}",
                         add_exception_stack_trace=True)
                 finally:
                     if not suite_success:
                         self._mark_log_as_failed()
 
-                next_check_log_start_time = datetime.datetime.utcnow()
-                suite_success = suite_success and self._check_agent_log_on_test_nodes(ignore_error_rules, check_log_start_time_override if check_log_start_time_override != datetime.datetime.max else check_log_start_time)
+                next_check_log_start_time = datetime.datetime.now(UTC)
+                suite_success = suite_success and self._check_agent_log_on_test_nodes(ignore_error_rules, check_log_start_time_override if check_log_start_time_override != datetime_max_utc else check_log_start_time)
 
                 return suite_success, next_check_log_start_time
 
@@ -762,7 +765,7 @@ class AgentTestSuite(LisaTestSuite):
                 # If there are multiple test nodes, as in a scale set, append the name of the node to the name of the result
                 test_result_name += '_' + node_name.split('_')[-1]
 
-            start_time: datetime.datetime = datetime.datetime.now()
+            start_time: datetime.datetime = datetime.datetime.now(UTC)
 
             try:
                 message = f"Checking agent log on test node {node_name}, starting at {check_log_start_time.strftime('%Y-%m-%dT%H:%M:%S.%fZ')}"
@@ -903,7 +906,7 @@ class AgentTestSuite(LisaTestSuite):
         msg.message = message
         if add_exception_stack_trace:
             msg.stacktrace = traceback.format_exc()
-        msg.elapsed = (datetime.datetime.now() - start_time).total_seconds()
+        msg.elapsed = (datetime.datetime.now(UTC) - start_time).total_seconds()
 
         notifier.notify(msg)
 
@@ -949,8 +952,12 @@ class AgentTestSuite(LisaTestSuite):
                         "publisher": "[parameters('publisher')]"
                     }
 
+        network_security_rule = NetworkSecurityRule(template, is_lisa_template=False)
+        # Disabling the default outbound access due to security requirement
+        log.info("******** Waagent: Marking subnet to disable default outbound access")
+        network_security_rule.disable_default_outbound_access()
         if self._allow_ssh != '':
-            NetworkSecurityRule(template, is_lisa_template=False).add_allow_ssh_rule(self._allow_ssh)
+            network_security_rule.add_allow_ssh_rule(self._allow_ssh)
 
         return template, {
             "username": {"value": self._user},
